/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws1871;

import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestSetup;
import junit.framework.Test;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceException;
import java.net.URL;

/**
 * Conformance (Handler resolver for a HandlerChain annotation): For a generated service class (see 2.7)
 * which is annotated with a HandlerChain annotation, the default handler resolver MUST return handler
 * chains consistent with the contents of the handler chain descriptor referenced by the HandlerChain annotation.
 */
public class JBWS1871TestCase extends JBossWSTest
{
   private final static String TNS =            "http://jbws1871.jaxws.ws.test.jboss.org/";
   private final static QName portName =        new QName(TNS, "JBWS1871EndpointImplPort");
   private final static QName serviceName =     new QName(TNS, "JBWS1871EndpointImplService");

   public static Test suite()
   {
      return new JBossWSTestSetup(JBWS1871TestCase.class, "jaxws-jbws1871.war");
   }

   public void testServiceWithoutHandlerChain() throws Exception
   {
      Service service = new JBWS1871EndpointImplService(
        new URL("http://"+getServerHost()+":8080/jaxws-jbws1871?wsdl"), serviceName 
      );
      JBWS1871Endpoint port = service.getPort(portName, JBWS1871Endpoint.class);

      String res = port.echo("testServiceWithoutHandlerChain");
      assertEquals(res, "testServiceWithoutHandlerChain");
   }

   public void testServiceWithHandlerChain() throws Exception
   {
      Service service = new HandlerChainService(
        new URL("http://"+getServerHost()+":8080/jaxws-jbws1871?wsdl"), serviceName
      );
      JBWS1871Endpoint port = service.getPort(portName, JBWS1871Endpoint.class);

      try
      {
         String res = port.echo("testServiceWithHandlerChain");
         fail("CausesAnExceptionHandler.class on HandlerChainService.class should throw an Exception upon invocation");
      } catch (WebServiceException e)
      {
         assertEquals("java.lang.RuntimeException: Intentionally created Exception", e.getMessage());
      }

   }
}
