package org.jboss.test.ws.jaxws.jbws2308;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.handler.Handler;

import org.jboss.wsf.common.DOMWriter;

public class ClientServlet extends HttpServlet
{
   private String endpointURL = "http://127.0.0.1:8080/jaxws-jbws2308/EndpointImpl";
   private String targetNS = "http://jbws2308.jaxws.ws.test.jboss.org/";
   
   @Override
   protected void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException
   {
      if (!testTransformer())
         throw new ServletException("Transformer test failed!");

      URL wsdlURL = new URL(endpointURL + "?wsdl");
      QName serviceName = new QName(targetNS, "EndpointService");
      Service service = Service.create(wsdlURL, serviceName);
      Endpoint port = (Endpoint)service.getPort(Endpoint.class);

      BindingProvider bindingProvider = (BindingProvider)port;
      List<Handler> handlerChain = new ArrayList<Handler>();
      handlerChain.add(new ClientHandler2());
      handlerChain.add(new ClientHandler3());
      handlerChain.add(new ClientHandler4());
      bindingProvider.getBinding().setHandlerChain(handlerChain);
      
      try
      {
         Object retObj = port.echo("Hello");
         throw new ServletException("A RuntimeException was expected");
      }
      catch (RuntimeException e)
      {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         PrintStream ps = new PrintStream(baos, true);
         e.printStackTrace(ps);
         String tmp = "ClientLogicalHandler6.handleMessage throws SOAPFaultException for ClientLogicalOutboundHandleFaultFalseTest";
         if (baos.toString().indexOf(tmp) > -1) {
           res.getOutputStream().println("Did get expected RuntimeException text");
         }
         else
         {
            System.out.println("Obtained this exception: " + baos.toString());
            throw new ServletException("Did not get expected RuntimeException text");
         }
      }
   }
   
   public boolean testTransformer()
   {
      try
      {
         TransformerFactory transformerFactory = TransformerFactory.newInstance();
         Transformer transformer = transformerFactory.newTransformer();
         DOMResult domResult = new DOMResult();
         String s = "<ns2:Fault xmlns:ns2=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
            "<faultcode xmlns:tns=\"myNamespace\">tns:ItsASoapFault</faultcode>" + 
            "<faultstring>ClientLogicalHandler6.handleMessage throws SOAPFaultException for ClientLogicalOutboundHandleFaultFalseTest</faultstring>" +
            "<faultactor>faultActor</faultactor>" + 
            "<detail>" + 
            "<somefaultentry/>" + 
            "</detail>" + 
            "</ns2:Fault>";
         ByteArrayInputStream bais = new ByteArrayInputStream(s.getBytes());
         StreamSource ss = new StreamSource(bais);
         
         transformer.transform(ss, domResult);
         
         return DOMWriter.printNode(domResult.getNode(), false).contains("myNamespace");
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }
      
   }
}
