/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws2308;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.handler.Handler;

import junit.framework.Test;

import org.jboss.wsf.common.DOMWriter;
import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestSetup;

public class JBWS2308TestCase extends JBossWSTest
{

   private String endpointURL = "http://" + getServerHost() + ":8080/jaxws-jbws2308/EndpointImpl";
   private String targetNS = "http://jbws2308.jaxws.ws.test.jboss.org/";

   public static Test suite()
   {
      return new JBossWSTestSetup(JBWS2308TestCase.class, "jaxws-jbws2308.jar");
   }

   public void testInvocation() throws Exception
   {
      URL wsdlURL = new URL(endpointURL + "?wsdl");
      QName serviceName = new QName(targetNS, "EndpointService");

      Service service = Service.create(wsdlURL, serviceName);
      Endpoint port = (Endpoint)service.getPort(Endpoint.class);

      BindingProvider bindingProvider = (BindingProvider)port;
      List<Handler> handlerChain = new ArrayList<Handler>();
      handlerChain.add(new ClientHandler2());
      handlerChain.add(new ClientHandler3());
      handlerChain.add(new ClientHandler4());
      bindingProvider.getBinding().setHandlerChain(handlerChain);
      
      try
      {
         Object retObj = port.echo("Hello");
         fail("A RuntimeException was expected");
      }
      catch (RuntimeException e)
      {
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         PrintStream ps = new PrintStream(baos, true);
         e.printStackTrace(ps);
         String tmp = "ClientLogicalHandler6.handleMessage throws SOAPFaultException for ClientLogicalOutboundHandleFaultFalseTest";
         if (baos.toString().indexOf(tmp) > -1) {
           System.out.println("Did get expected RuntimeException text");
         }
         else
         {
            fail("Did not get expected RuntimeException text");
         }
      }
      
   }
   
   
   public void testTransformer() throws Exception
   {
      TransformerFactory transformerFactory = TransformerFactory.newInstance();
      Transformer transformer = transformerFactory.newTransformer();
      DOMResult domResult = new DOMResult();
      String s = "<ns2:Fault xmlns:ns2=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
         "<faultcode xmlns:tns=\"myNamespace\">tns:ItsASoapFault</faultcode>" + 
         "<faultstring>ClientLogicalHandler6.handleMessage throws SOAPFaultException for ClientLogicalOutboundHandleFaultFalseTest</faultstring>" +
         "<faultactor>faultActor</faultactor>" + 
         "<detail>" + 
         "<somefaultentry/>" + 
         "</detail>" + 
         "</ns2:Fault>";
      ByteArrayInputStream bais = new ByteArrayInputStream(s.getBytes());
      StreamSource ss = new StreamSource(bais);
      
      transformer.transform(ss, domResult);
      
      assertTrue(DOMWriter.printNode(domResult.getNode(), false).contains("myNamespace"));
   }
   
}

